import os
import os.path as xpath

from lk_utils.filesniff import currdir
from lk_utils.filesniff import normpath

from ._env import ASSETS_ENTRY  # str['STANDALONE', 'PACKAGE']

print(':v1', ASSETS_ENTRY)

_cur_dir = currdir()


def src_2_dst(src_path, src_dir='', dst_dir=''):
    if not src_path:
        return ''
    if src_path.startswith('dist:'):
        return src_path.replace('dist:', dst_model.dst_root)
    
    if not src_dir: src_dir = src_model.src_root
    if not dst_dir: dst_dir = dst_model.src_root
    
    p = relpath(src_path, src_dir)
    p = f'{dst_dir}/{p}'
    return p


def relpath(path, start):
    if not path:
        return ''
    return normpath(xpath.relpath(path, start))


class PyPortablePathModel:
    # current dir based
    cur_root = _cur_dir
    print(':v1', cur_root)
    
    (
        bat_2_exe_converter,
        checkup,
        template,
    ) = (
        f'{cur_root}/bat_2_exe/bat_to_exe_converter.exe',
        f'{cur_root}/checkup',
        f'{cur_root}/template',
    )
    
    (
        compilers_lib,
        pyportable_runtime_py38,
        pyportable_runtime_py39,
        pyportable_runtime_py310,
        pyportable_runtime_py38_linux,
    ) = (
        f'{cur_root}/compilers/lib',
        f'{cur_root}/compilers/lib/pyportable_runtime_py38/pyportable_runtime',
        f'{cur_root}/compilers/lib/pyportable_runtime_py39/pyportable_runtime',
        f'{cur_root}/compilers/lib/pyportable_runtime_py310/pyportable_runtime',
        f'{cur_root}/compilers/lib/linux/pyportable_runtime_py38/pyportable_runtime',
    )
    
    (
        launch_bat,
        pyarmor,
        pylauncher,
        pyproject,
        python_ico,
        pytransform,
    ) = (
        f'{cur_root}/template/launch.bat',
        f'{cur_root}/template/pyarmor',
        f'{cur_root}/template/pylauncher.txt',
        f'{cur_root}/template/pyproject.json',
        f'{cur_root}/template/python.ico',
        f'{cur_root}/template/pytransform.txt',
    )
    
    (
        depsland_launcher_part_a,
        depsland_launcher_part_b,
        depsland_setup_part_a,
        depsland_setup_part_b,
    ) = (
        f'{cur_root}/template/depsland/launcher_part_a.bat',
        f'{cur_root}/template/depsland/launcher_part_b.bat',
        f'{cur_root}/template/depsland/setup_part_a.bat',
        f'{cur_root}/template/depsland/setup_part_b.txt',
    )
    
    # --------------------------------------------------------------------------
    # project root
    
    if ASSETS_ENTRY == 'STANDALONE':
        prj_root = xpath.dirname(cur_root)
    else:
        prj_root = f'{cur_root}/assets'
    
    (
        dist,
        # lib,
        temp,
    ) = (
        f'{prj_root}/dist',
        # f'{prj_root}/lib',
        f'{prj_root}/temp',
    )
    
    # --------------------------------------------------------------------------
    
    def build_dirs(self):
        if ASSETS_ENTRY == 'STANDALONE':
            assert xpath.exists(self.dist)
            assert xpath.exists(self.temp)
        else:
            if not xpath.exists(self.prj_root):
                os.mkdir(self.prj_root)
                os.mkdir(self.dist)
                os.mkdir(self.temp)
    
    def create_temp_dir(self):
        from secrets import token_urlsafe
        _temp_dir = f'{self.temp}/{token_urlsafe()}'
        print('make temporary dir', _temp_dir)
        os.mkdir(_temp_dir)
        return _temp_dir


class SourcePathModel:
    src_root = None
    prj_root = None
    
    # noinspection PyAttributeOutsideInit
    def init(self, src_root, prj_root, **kwargs):
        self.src_root = src_root
        self.prj_root = prj_root
        
        self.readme = kwargs.get('readme', '')
    
    def assert_ready(self):
        assert self.src_root and self.prj_root


class DistributedPathModel:
    """
    Tree:
        dist
        |= hello_world_0.1.0    # dst_root
        |                       # ↑ this node concept doesn't exist in `src_model`
            |= src              # src_root
            |                   # ↑ this is correspondent to `src_model.src_root`
                |= hello_world  # prj_root
                |               # ↑ this is correspondent to `src_model.prj_root`
                    |- main.py
                |- pylauncher.py
    """
    dst_root = None
    src_root = None
    prj_root = None
    
    # noinspection PyAttributeOutsideInit
    def init(self, dst_root: str, prj_relroot: str, **kwargs):
        self.dst_root = dst_root
        self.src_root = f'{self.dst_root}/src'
        self.prj_root = f'{self.src_root}/{prj_relroot}'
        #   prj_relroot: came from
        #       `relpath(src_model.prj_root, src_model.src_root)`
        
        # prj_root/*
        self.pylauncher = f'{self.src_root}/pylauncher.py'
        self.pylauncher_conf = f'{self.src_root}/.pylauncher_conf'
        
        # dst_root/*
        self.build = f'{self.dst_root}/build'
        self.lib = f'{self.dst_root}/lib'
        self.src = f'{self.dst_root}/src'
        self.venv = f'{self.dst_root}/venv'
        
        # dst_root/*/*
        self.manifest = f'{self.build}/manifest.json'
        self.python = f'{self.venv}/python.exe'
        
        # other: depsland
        self.depsland_setup_part_a = f'{self.dst_root}/setup.bat'
        self.depsland_setup_part_b = f'{self.build}/setup.py'
        
        # other: kwargs
        launcher_name = kwargs['launcher_name']
        self.launcher_bat = f'{self.dst_root}/{launcher_name}.bat'
        self.launcher_exe = f'{self.dst_root}/{launcher_name}.exe'
        self.readme = src_2_dst(kwargs.get('readme', ''))
    
    def assert_ready(self):
        assert self.dst_root and self.src_root and self.prj_root


prj_model = PyPortablePathModel()
src_model = SourcePathModel()
dst_model = DistributedPathModel()

prj_model.build_dirs()

__all__ = [
    'src_2_dst', 'relpath',
    'prj_model', 'src_model', 'dst_model'
]



import os
import sys
import traceback

from importlib import import_module
from os.path import abspath


def _add_paths(paths, **special_paths):
    import os
    import sys
    
    os.chdir(os.path.dirname(__file__))

    # add current dir to `sys.path` (this is necessary for embed python)
    sys.path.insert(0, abspath('.'))
    sys.path.insert(1, abspath(special_paths['project_lib']))
    sys.path.extend(map(os.path.abspath, paths))
    
    if special_paths.get('pywin32'):
        # ref: https://stackoverflow.com/questions/58612306/how-to-fix
        #      -importerror-dll-load-failed-while-importing-win32api
        #      (see comment of @ocroquette)
        try:
            import pywin32_system32
        except ImportError:
            print('Adding pywin32 support failed')
            
        _pywin32_dir = pywin32_system32.__path__._path[0]  # noqa
        #   -> ~/lib/site-packages/pywin32_system32
        os.add_dll_directory(_pywin32_dir)
        #   WARNING: this method is available since python 3.8.
        
        _site_packages_dir = os.path.dirname(_pywin32_dir)
        sys.path.extend((
            _site_packages_dir + '/pythonwin',
            _site_packages_dir + '/win32',
            _site_packages_dir + '/win32/lib',
        ))


# ------------------------------------------------------------------------------


def auth_launch():
    pass


def launch(main_func, *args, **kwargs):
    try:
        main_func(*args, **kwargs)
    except Exception:
        _show_error_info(traceback.format_exc())
        input('Press enter to leave...')
    sys.exit()


def _show_error_info(err_msg, title='Runtime Exception', terminal='console'):
    """
    Args:
        err_msg: suggest passing `traceback.format_exc()`.
        title:
        terminal:

    Rerferences:
        https://stackoverflow.com/questions/1278705/when-i-catch-an-exception
            -how-do-i-get-the-type-file-and-line-number
        https://stackoverflow.com/questions/17280637/tkinter-messagebox-without
            -window
        https://www.cnblogs.com/freeweb/p/5048833.html
    """
    if terminal == 'console':
        print(title + ':', err_msg)
    elif terminal == 'tkinter':
        from tkinter import Tk, messagebox
        root = Tk()
        root.withdraw()
        messagebox.showerror(title=title, message=err_msg)
    elif terminal == 'vbsbox':
        return os.popen(
            'echo msgbox "{{msg}}", 64, "{{title}}" > '
            'alert.vbs && start '
            'alert.vbs && ping -n 2 127.1 > '
            'nul && del alert.vbs'.format(title=title, msg=err_msg)
        ).read()


if __name__ == '__main__':
    import os

    os.chdir(os.path.dirname(__file__))

    try:
        conf = _parse_target_conf(sys.argv)
        
        # check in target dir to make sure all sequent relative paths
        # references are based on the target dir.
        os.chdir(conf['TARGET_DIR'])
        sys.path.append(abspath('.'))
        
        mod = import_module(conf['TARGET_MOD'], conf['TARGET_PKG'])
        if conf['TARGET_FUNC']:
            main = getattr(mod, conf['TARGET_FUNC'])
            launch(main, *conf['TARGET_ARGS'], **conf['TARGET_KWARGS'])
    
    except Exception:
        _show_error_info(traceback.format_exc())
        input('Press enter to leave...')



import os
import shutil

from lk_utils import dumps
from lk_utils import loads

from .base_compiler import BaseCompiler
from ..global_conf import gconf
from ..path_model import dst_model
from ..path_model import prj_model
from ..path_model import src_model


class PyportableCompiler(BaseCompiler):
    
    # noinspection PyMissingConstructor
    def __init__(self, salt: str, **_):
        """
        Args:
            salt: str.
                PyPortableCompiler has its own private key, so the param is
                receiving a salt value.
                the private key is hidden in `.lib.*.pyportable_runtime.cipher`,
                which is a '.pyd' or '.so' file. more infomation see `~/sidework
                /generate_pyportable_crypto_trial_version.py`.
            **_:
        """
        print(':v', '[D3100]', salt)
        self.salt = salt
        
        from textwrap import dedent
        self._template = dedent('''
            from pyportable_runtime import decrypt
            globals().update(decrypt({ciphertext}, globals(), locals()))
        ''').strip()
        
        self._generate_runtime_lib()
    
    def _generate_runtime_lib(self):
        import pyportable_runtime  # noqa
        #   this package is from `.lib.*.pyportable_runtime`.
        #   the package path is added since `../main_flow/step1/init_key_params
        #   .py:[func]_init_pyportable_runtime_package`.
        
        if gconf.target_platform == 'windows':
            package_dir = {
                'python38' : prj_model.pyportable_runtime_py38,
                'python39' : prj_model.pyportable_runtime_py39,
                'python310': prj_model.pyportable_runtime_py310,
            }[gconf.target_pyversion]
            #   note this is different from `../main_flow/step1/init_key_params
            #   .py:[func]_init_pyportable_runtime_package`. here we are using
            #   `gconf.target_pyversion`, not `gconf.current_pyversion`.
        else:
            package_dir = prj_model.pyportable_runtime_py38_linux
        
        dir_i = package_dir
        dir_o = dst_model.lib + '/pyportable_runtime'
        
        #: [scheme 1]
        shutil.copytree(dir_i, dir_o)
        
        #: [scheme 2] copy whole dir_i to dir_o, but exclude __pycache__ folder.
        # os.mkdir(dir_o)
        # for name in os.listdir(dir_i):
        #     if name == '__pycache__': continue
        #     shutil.copyfile(f'{dir_i}/{name}', f'{dir_o}/{name}')
        
        # add salt file ('<dir_o>/__salt__')
        dumps(pyportable_runtime.encrypt(self.salt).decode('utf-8'),
              dir_o + '/__salt__')
    
    def compile_all(self, pyfiles):
        print(':i0')
        for i, o in pyfiles:
            print(':i', 'compiling', os.path.relpath(i, src_model.prj_root))
            self.compile_one(i, o)
    
    def compile_one(self, src_file, dst_file):
        if os.path.basename(src_file) == '__init__.py':
            shutil.copyfile(src_file, dst_file)
            return dst_file
        
        data_r = loads(src_file)
        data_w = self._encrypt(data_r)
        dumps(data_w, dst_file)
        
        return dst_file
    
    def _encrypt(self, data: str) -> str:
        from pyportable_runtime import encrypt  # noqa
        data = encrypt(data, add_shell=True, salt=self.salt)  # type: bytes
        return self._template.format(ciphertext=data)



if True:
    import os
    import sys
    
    os.chdir(os.path.dirname(__file__))
    sys.path.append('../../')

try:
    from pyportable_installer import full_build
except ImportError:
    input('ImportError: pyportable_installer')
    sys.exit(1)

try:
    _, file = sys.argv
    assert os.path.exists(file)
except (AssertionError, ValueError):
    input('You must pass the file (like "~/pyproject.json" and make sure the '
          'file exists.')
    sys.exit(1)

try:
    result = full_build(file)
except Exception:
    from traceback import format_exc
    
    input(format_exc())
    sys.exit(1)
else:
    input('Packaging successful! See result at {}'.format(
        result['build']['dist_dir']
    ))
    sys.exit(0)



import re
from os.path import exists

from lk_utils import run_cmd_args

from ...global_conf import gconf
from ...path_model import prj_model


def get_full_python(pyversion):
    """
    Args:
        pyversion: str['python38', 'python39', 'python310']
    
    Returns:
        str: python executable path. note this is file path, not dir.
    """
    pyversion = '.'.join(re.search(r'(\d)(\d+)', pyversion).groups())
    # -> '3.8', '3.9', '3.10'
    if gconf.current_platform == 'windows':
        path = run_cmd_args(
            'py', f'-{pyversion}', '-c', 'import sys; print(sys.executable)'
        )
        if ' not found!' in path:
            ''' e.g.
                Python 3.11 not found!
                Installed Pythons found by py Launcher for Windows
                 -3.10-64 *
                 -3.9-64
                 -3.8-64
                 -2.7-64
            '''
            raise FileNotFoundError(path)
        return path
    else:
        path = f'/usr/bin/python{pyversion}'
        if not exists(path):
            raise FileNotFoundError(path)
        return path


def get_embed_python(pyversion, add_pip_suits: bool):
    from embed_python_manager import EmbedPythonManager
    manager = EmbedPythonManager(pyversion)
    manager.change_source('npm_taobao_org.yml')
    manager.deploy(
        add_pip_suits=add_pip_suits,
        add_pip_scripts=False,
        add_tk_suits=False  # FIXME: check tkinter requirement
    )
    return manager.python


def get_embed_python_from_local(pyversion):
    if pyversion == 'python38' and \
            exists(f'{prj_model.prj_root}/venv/python38._pth.bak'):
        return f'{prj_model.prj_root}/venv/python.exe'
    else:
        return ''



import os
import os.path

from . import where_is_python
from ...global_conf import gconf
from ...path_model import prj_model
from ...typehint import *

_conf = ...  # type: TConf


def init_key_params(conf: TConf, **kwargs):
    global _conf
    _conf = conf
    
    _init_proj_paths(**kwargs)
    _init_attachments_scheme()
    _init_pyversion()
    _init_platform()
    _init_pyportable_runtime_package()
    _init_python_paths()


def _init_proj_paths(**kwargs):
    gconf.pyproj_dir = kwargs['pyproj_dir']
    gconf.pyproj_file = kwargs['pyproj_file']


def _init_attachments_scheme():
    assert type(_conf['build']['attachments_exclusions']) is tuple
    gconf.attachments_exclusions = _conf['build']['attachments_exclusions']
    gconf.attachments_exist_scheme = _conf['build']['attachments_exist_scheme']


def _init_pyversion():
    pyversion = _conf['build']['python_version']
    assert pyversion, '`conf["build"]["python_version"]` not defined'
    pyversion = 'python' + pyversion.replace('.', '')  # e.g. 'python38'
    gconf.target_pyversion = pyversion


def _init_platform():
    p = _conf['build']['experimental_features']['platform']
    if p == 'system_default':
        from platform import system
        p = system().lower()  # 'windows', 'linux', 'macos'.
    
    assert p in ('windows', 'linux', 'macos'), 'platform not support'
    
    gconf.target_platform = p


def _init_pyportable_runtime_package():
    import sys
    
    if gconf.target_platform != 'windows':
        # # package_dir = {
        # #     'python38' : prj_model.pyportable_runtime_py38_linux,
        # #     'python39' : prj_model.pyportable_runtime_py39_linux,
        # #     'python310': prj_model.pyportable_runtime_py310_linux,
        # # }[gconf.target_pyversion]
        try:
            assert _conf['build']['python_version'] == '3.8'
        except AssertionError:
            raise Exception('For Linux/macOS system, the python version '
                            'accepts only 3.8 for now.')
        try:
            assert os.path.exists(prj_model.pyportable_runtime_py38_linux)
        except AssertionError:
            from textwrap import dedent
            print(':d', 'interactive prompt')
            runtime_dir = input(dedent('''
                The platform you defined in project configurations requires
                pyportable_runtime package which is built on Linux. You can find
                this package in https://github.com/likianta/pyportable-installer
                (browse ~/addons/linux/pyportable_runtime_py38), and copy it
                here:
                    {}
                Then re-run this installer.
                BTW if you have downloaded that package but not copied, you can
                pass me the path of download here, to continue the process (or
                press enter to leave):
            '''.format(
                prj_model.pyportable_runtime_py38_linux
            )).strip() + ' ').strip()
            if runtime_dir == '':
                exit()
            else:
                assert os.path.exists(runtime_dir)
                prj_model.pyportable_runtime_py38_linux = runtime_dir
    
    # -------------------------------------------------------------------------
    
    if gconf.current_platform == 'windows':
        runtime_dir = {
            'python38' : prj_model.pyportable_runtime_py38,
            'python39' : prj_model.pyportable_runtime_py39,
            'python310': prj_model.pyportable_runtime_py310,
        }[gconf.current_pyversion]
        sys.path.insert(0, os.path.dirname(runtime_dir))
    else:
        sys.path.insert(0, prj_model.pyportable_runtime_py38_linux)
    
    import pyportable_runtime  # noqa
    print(':v2', '[I4053]', pyportable_runtime.__path__)


def _init_python_paths():
    """
    
    Requirements:
        | venv/compiler     | full python | embed python | pip |
        | ----------------- | ----------- | ------------ | --- |
        | cython            |      Y      |              |     |
        | mypyc             |      Y      |              |     |
        | nuitka            |      Y      |              |     |
        | pip               |             |      Y       |  Y  |
        | pyarmor           |             |      Y       |     |
        | pyc               |             |      Y       |     |
        | pyportable_crypto |             |              |     |
        | source_venv       |             |      Y       |     |
        | *zipapp           |             |              |  Y  |
    
        Notes:
            - zipapp is not supported.
            - the venv/compiler list is from `typehint._TCompilerOptions` and
              `typehint._TVenvModeOptions`.
            - venv related options only available when enable_venv is True.
            - pyportable_crypto is the best option in current version.
            - the table is made for general suggestions, not responsible for the
              final decision.
    """
    name = _conf['build']['compiler']['name']  # type: TCompilerName
    mode = _conf['build']['venv']['mode']  # type: TVenvMode
    enable_venv = _conf['build']['venv']['enable_venv']  # type: bool
    
    # --------------------------------------------------------------------------
    
    is_full_python_required = False
    is_embed_python_required = False
    is_pip_required = False
    
    # 1. is_full_python_required
    if not enable_venv:
        is_full_python_required = True
    elif name in ('cython', 'mypyc', 'nuitka'):
        is_full_python_required = True
    
    # 2. is_embed_python_required
    if (
            (name in ('pyarmor', 'pyc')) or
            (enable_venv and mode == 'pip') or
            (enable_venv and mode == 'source_venv' and
             _conf['build']['venv']['options']['source_venv']['copy_venv'])
    ):
        is_embed_python_required = True
    
    # 3. is_pip_required
    if enable_venv and mode == 'pip':
        is_pip_required = True
    
    print(':v2', '[I5239]',
          is_full_python_required,
          is_embed_python_required,
          is_pip_required)
    
    # --------------------------------------------------------------------------
    
    pyversion = gconf.target_pyversion
    
    if is_full_python_required:
        # noinspection PyTypedDict
        options = _conf['build']['compiler']['options'][name]
        if options.get('python_path', '') in \
                ('{auto_detect}', 'auto_detect', ''):
            try:
                gconf.full_python = where_is_python.get_full_python(pyversion)
            except FileNotFoundError:
                print(
                    ':v3', '[W5951]',
                    'Cannot find installed python in your computer. Please '
                    'pass in your python path (e.g. ~/python.exe) manually.'
                )
                gconf.full_python = input(
                    f'Input file path here ({pyversion}): '
                )
        else:
            gconf.full_python = options['python_path']
        print(':v2', '[I1955]', gconf.full_python)
        assert os.path.exists(gconf.full_python)
    
    # --------------------------------------------------------------------------
    
    if is_embed_python_required:
        if is_pip_required:
            gconf.embed_python = where_is_python.get_embed_python(
                pyversion, add_pip_suits=True
            )
        else:
            gconf.embed_python = (
                    where_is_python.get_embed_python_from_local(pyversion) or
                    where_is_python.get_embed_python(
                        pyversion, add_pip_suits=False
                    )
            )
        assert os.path.exists(gconf.embed_python)



import sys
from textwrap import dedent

from pyportable_installer import *


def main():
    print(dedent('''\
        Command template:
            [mode] <pyproject_file>
            
        For example:
            0 D:\\my_project\\hello_world\\pyproject.json
            
        Params:
            mode:
                0   full build (default)
                1   min build
                2   debug build
            pyproject_file:
                An absolute path to your 'pyproject.json'.
                The filename is changeable, but please make sure it is placed
                right in its project and all relative paths description in it
                is calculated based on its directory.
    '''))
    
    while True:
        cmd = input('Your command here: ')
        if cmd == '':
            continue
        elif cmd == 'x':
            sys.exit()
        else:
            break
    
    if cmd.startswith(('0 ', '1 ', '2 ')):
        mode, file = cmd.split(' ', 1)
    else:
        mode, file = '0', cmd
    
    if mode == '0':
        build = full_build
    elif mode == '1':
        build = min_build
    elif mode == '2':
        build = debug_build
    else:
        raise ValueError(mode)
    
    build(file)


if __name__ == '__main__':
    main()



"""
requirements:
    pip install pysimplegui
"""
import os.path

import PySimpleGUI as Gui

import pyportable_installer

Gui.theme('SandyBeach')


class GuiEx:
    """ extended gui components. """
    
    # noinspection PyPep8Naming
    @staticmethod
    def FormGroup(title='', data: dict = None, key: str = None,
                  use_prefix=True):
        # data: dict[str key, str value]
        
        if use_prefix and key:
            prefixify = lambda k: f'{key}.{k}'
        else:
            prefixify = lambda k: k
        
        _best_key_width = max(map(len, data.keys()))
        
        def unpack_dict():
            for k, v in data.items():
                yield [  # this is a row
                    Gui.Text(k, size=(_best_key_width, 1),
                             justification='right'),
                    Gui.Input(default_text=v, key=prefixify(k))
                ]
        
        return Gui.Frame(title, list(unpack_dict()), key=key)


def main():
    layout = [
        [  # row 0: welcome message
            Gui.Text('Welcome using PyPortable Installer!\n'
                     'Input a pyproject.json file to start...'),
            # Gui.Text('Input a pyproject.json file to start...'),
        ],
        
        [  # row 1: pyproj conf file input
            Gui.Input(change_submits=True, key='file'),
            Gui.FileBrowse('Browse', key='file_browse'),
        ],
        
        [  # row 2: most modified configurations, change them here will
            # override the ones in the pyproj conf file.
            GuiEx.FormGroup('Most modified options', {
                'app_name'   : '',
                'app_version': '',
                'proj_dir'   : '',
                'dist_dir'   : '',
            }, key='options')
        ],
        
        [  # row 3: buttons
            Gui.Button('Reload', key='reload'),
            Gui.Button('Overwrite', key='overwrite'),
            Gui.Button('Run', key='run'),
            Gui.Text('', key='short_info')
        ],
        
        [  # row 4: open result
            Gui.Text('', key='result'),
            Gui.Button('Open', visible=False, key='result_open')
        ]
    ]
    
    win = Gui.Window('PyPortable Installer', layout)
    mainloop(win)


def mainloop(win: Gui.Window):
    import os.path
    
    pyproj_conf = PyProjConf()
    controls = {'get_ready_to_run': False}
    
    while True:
        evt, val = win.read()
        # lk.logp(evt, val)
        
        if evt in (None, 'Exit'):
            break
        
        elif evt == 'file':
            file = val[evt]
            if file == pyproj_conf.file:
                continue
            else:
                # FIXME:
                #   win['file_browse'].update(initial_folder=os.path.dirname(file))
                win['result'].update('')
                win['result_open'].update(visible=False)
            
            if file.endswith('.json') and os.path.exists(file):
                pyproj_conf.load(file)
                try:
                    win['options.app_name'].update(pyproj_conf.app_name)
                    win['options.app_version'].update(pyproj_conf.app_version)
                    win['options.proj_dir'].update(pyproj_conf.proj_dir)
                    win['options.dist_dir'].update(pyproj_conf.dist_dir)
                    win['short_info'].update('Pyproj file loaded.')
                    controls['get_ready_to_run'] = True
                except KeyError:
                    win['short_info'].update('Invalid pyproj file!')
                    controls['get_ready_to_run'] = False
        
        elif evt == 'reload':
            pyproj_conf.load(pyproj_conf.file)
            win['options.app_name'].update(pyproj_conf.app_name)
            win['options.app_version'].update(pyproj_conf.app_version)
            win['options.proj_dir'].update(pyproj_conf.proj_dir)
            win['options.dist_dir'].update(pyproj_conf.dist_dir)
            win['short_info'].update('Reload successful!')
        
        elif evt == 'overwrite':
            pyproj_conf.overwrite({
                'app_name'   : win['options.app_name'].get(),
                'app_version': win['options.app_version'].get(),
                'proj_dir'   : win['options.proj_dir'].get(),
                'dist_dir'   : win['options.dist_dir'].get(),
            })
            win['short_info'].update('Modification saved!')
        
        elif evt == 'run':
            if not controls['get_ready_to_run']:
                win['short_info'].update(
                    'Program is not ready to run. Please check your '
                    'configuration.'
                )
                continue
            try:
                result = pyportable_installer.full_build(
                    val['file'], additional_conf={
                        'app_name'   : win['options.app_name'].get(),
                        'app_version': win['options.app_version'].get(),
                        'build'      : {
                            'proj_dir': win['options.proj_dir'].get(),
                            'dist_dir': win['options.dist_dir'].get(),
                        }
                    }
                )
            except Exception as e:  # noqa
                # import sys
                from traceback import format_exc
                # from traceback import format_exception
                err_win = Gui.Window('Build Failed', [
                    [Gui.Text(format_exc())],
                ])
                while True:
                    evt, _ = err_win.read()
                    if evt in (None, 'Exit'):
                        break
                win['short_info'].update('')
                continue
            
            win['short_info'].update('Packaging done!')
            win['result'].update(result['build']['dist_dir'])
            win['result_open'].update(visible=True)
        
        elif evt == 'result_open':
            path = win['result'].get()
            if os.path.exists(path):
                from lk_utils import run_cmd_args
                run_cmd_args('start', path)
                #   note: do not use `explorer <path>`, it will raise an error.
                #        (ps: the error message shows empty, so i don't known
                #         why this happens.)
                win['short_info'].update('')
            else:
                win['short_info'].update('Target dir not exists!')


class PyProjConf:
    from pyportable_installer.typehint import TConf
    file: str = ''
    dir_: str = ''
    conf: TConf = None
    
    def load(self, file):
        from lk_utils import loads
        self.file = file
        self.dir_ = os.path.dirname(file)
        self.conf = loads(file)
    
    reload = load
    
    @property
    def app_name(self):
        return self.conf['app_name']
    
    @property
    def app_version(self):
        return self.conf['app_version']
    
    @property
    def proj_dir(self):
        return self.conf['build']['proj_dir']
    
    @property
    def dist_dir(self):
        # note: the dist_dir is not the really existed in the system. because
        #   it contains '../' (which is relative to pyporj file), and some
        #   placeholders (e.g. '{app_name}', '{app_name_lower}',
        #   '{app_version}', etc.).
        return self.conf['build']['dist_dir']
    
    def overwrite(self, options):
        self.conf['app_name'] = options['app_name']
        self.conf['app_version'] = options['app_version']
        self.conf['build']['proj_dir'] = options['proj_dir']
        self.conf['build']['dist_dir'] = options['dist_dir']
        from lk_utils import dumps
        dumps(self.conf, self.file, pretty_dump=True)
        return True


if __name__ == '__main__':
    main()



from pathlib import Path

try:
    from genexe.generate_exe import generate_exe
except ImportError:
    generate_exe = None
from lk_utils.read_and_write import load_list
from lk_utils.subproc import format_cmd  # noqa


def bat_2_exe(file_i, file_o, icon='', show_console=True):
    if generate_exe is None:
        print('cannot generate exe from bat: gen-exe is not installed', ':v4p')
        print('^ if you see this info in windows, plese `pip install gen-exe` '
              'then try again.', ':v2p')
        return
    
    data_r = load_list(file_i)
    # data_w = ' && '.join(format_cmd(*data_r))
    data_w = ' && '.join(data_r).strip()
    
    # https://github.com/silvandeleemput/gen-exe
    # https://blog.csdn.net/qq981378640/article/details/52980741
    data_w = data_w.replace('%~dp0', '{EXE_DIR}')
    data_w = data_w.replace('%cd%', '{EXE_DIR}')
    if data_w.endswith('%*'): data_w = data_w[:-3]
    
    generate_exe(
        target=Path(file_o),
        command=data_w,
        icon_file=Path(icon),
        show_console=show_console
    )



encrypt = None
decrypt = None


def _init_check():
    import sys

    current_pyversion = sys.version_info[:2]  # type: tuple
    target_pyversion = (3, 9)
    if current_pyversion != target_pyversion:
        raise Exception(
            "Python interpreter version doesn't matched!",
            "Required: Python {}, got {} ({})".format(
                ', '.join(map(str, target_pyversion)),
                ', '.join(map(str, current_pyversion)),
                sys.executable
            )
        )


_init_check()
del _init_check

from .cipher import decrypt  # noqa
from .cipher import encrypt  # noqa

__version__ = '1.0.0'



class NameConverter:
    
    def __init__(self, raw_name):
        # raw_name is whitespace separated string.
        self._name = raw_name
    
    @property
    def raw(self):
        return self._name
    
    # natual
    
    @property
    def lower_case(self):
        return self._name.lower()
    
    @property
    def upper_case(self):
        return self._name.upper()
    
    @property
    def title_case(self):
        return self._name.title()
    
    # programming
    
    @property
    def snake_case(self):
        return self._name.lower().replace(' ', '_')
    
    @property
    def kebab_case(self):
        return self._name.lower().replace(' ', '-')
    
    @property
    def pascal_case(self):
        return self._name.title().replace(' ', '')
    
    @property
    def camel_case(self):
        if ' ' not in self._name:
            return self._name.lower()
        a, b = self._name.split(' ', 1)
        return a.lower() + b.title()



import sys


def parse_sys_argv():
    """
    Input (example):
        sys.argv = ['main.py', 'conf.pkl', 'launcher.exe', 'hello', 'world']
    Process:
        - Detect exe file and remove it from sys.argv.
        - Load config file.
        - The real arguments are ['hello', 'world'].
    """
    args = sys.argv
    #   the first arg is python filename to launch, the second is preset
    #   configuration file, the third is optional exe file.
    
    if len(args) == 0:
        raise ValueError('Unreachable case, the arguments cannot be none!')
    elif len(args) == 1:
        # WARNING: this won't be supported in the future.
        conf_file = '__main__'
        real_args = []
    else:
        if idx := _check_exe_file_in_args(args):
            args.pop(idx)
        conf_file = args.pop(1)
        real_args = args
    
    conf = _load_config(conf_file)
    return conf, real_args


def configure_launch_args():
    pass


def _check_exe_file_in_args(args):
    target_exe_dir = os.path.dirname(os.getcwd())
    if (len(args) > 2
            and args[2].endswith('.exe')
            and os.path.dirname(args[2]) == target_exe_dir):
        return 2
    else:
        return 0


def _load_conf(conf_file):
    assert conf_file.endswith(('.json', '.pkl')), (
        'Unknown file type to load config!', conf_file
    )
    
    if conf_file.endswith('.json'):
        from json import load
        with open(conf_file, 'r', encoding='utf-8') as f:
            return load(f)
    else:
        from pickle import load
        with open(conf_file, 'rb') as f:
            return load(f)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyportable_installer',
 'pyportable_installer.checkup',
 'pyportable_installer.compilers',
 'pyportable_installer.compilers.lib.pyportable_runtime_py310.pyportable_runtime',
 'pyportable_installer.compilers.lib.pyportable_runtime_py38.pyportable_runtime',
 'pyportable_installer.compilers.lib.pyportable_runtime_py39.pyportable_runtime',
 'pyportable_installer.main_flow',
 'pyportable_installer.main_flow.step1',
 'pyportable_installer.main_flow.step2',
 'pyportable_installer.main_flow.step3',
 'pyportable_installer.main_flow.step3.step3_1',
 'pyportable_installer.main_flow.step3.step3_2',
 'pyportable_installer.main_flow.step3.step3_3',
 'pyportable_installer.main_flow.step4',
 'pyportable_installer.template.pylauncher',
 'pyportable_installer.user_interface']

package_data = \
{'': ['*'],
 'pyportable_installer': ['template/*', 'template/depsland/*'],
 'pyportable_installer.user_interface': ['qml/*']}

install_requires = \
['embed-python-manager',
 'lk-logger>=5.0.1,<6.0.0',
 'lk-utils>=2.2.0,<3.0.0',
 'pyportable-crypto>=1.0.0,<2.0.0']

extras_require = \
{':platform_system == "Windows"': ['gen-exe'],
 'all': ['pillow', 'pyarmor', 'pysimplegui'],
 'gui': ['pysimplegui']}

setup_kwargs = {
    'name': 'pyportable-installer',
    'version': '4.3.1',
    'description': 'Build and distribute portable Python application by all-in-one configuration file.',
    'long_description': '# PyPortable Installer\n\n> pyportable-installer 提供源码版和直装版两个版本供使用. 请根据需要选择:\n>\n> | 版本类型 | 说明 | 体积 | 下载方式 |\n> | -------- | ---- | ---- | -------- |\n> | 第三方库 | 适合开发者在 Python 工程中引入 | 4MB | `pip install pyportable-installer` |\n> | 独立免安装版 (标准版) | 用户电脑上需预先安装 [Depsland][1] 软件, 安装过程需要联网 | 4.5MB | 见本项目 Release 页面 |\n> | 独立免安装版 (完整版) | 该压缩包内置了 [Depsland][1]. 下载后解压即可使用, 无需 Python 开发环境, 完全离线安装 | 47MB | 见本项目 Release 页面 |\n>\n> `python -m pyportable_installer` 快速演示:\n>\n> ![](./.assets/QCdvnW6kJK.gif)\n\n--------------------------------------------------------------------------------\n\n`pyportable-installer` 是一个 Python 项目打包工具, 它受启发于 [poetry](https://github.com/python-poetry/poetry), 并旨在作为 [pyinstaller](https://github.com/pyinstaller/pyinstaller) 的替代品出现.\n\n`pyportable-installer` 通过一个 **all-in-one 配置文件** 来管理打包工作, 通过该文件可将您的 Python 项目打包为 "免安装版" 的软件, 用户无需安装 Python 程序或第三方依赖 (注: 该特性需要您在配置中启用虚拟环境选项), 真正做到 "开箱即用, 双击启动".\n\n# 特性\n\n`pyportable-installer` 具有以下特点:\n\n1.  打包后的体积很小. 在不附带虚拟环境的情况下, 与您的源代码同等量级 (这通常只有几百 KB ~ 几 MB 之间)\n\n2.  易于使用. 您只需要维护一个 pyproject.json 配置文件即可. 在快速迭代的环境下, 您甚至只需要更改版本号就能立即生成新的打包结果\n\n3.  打包速度快. 一个中小型项目在数秒间即可生成打包结果\n\n4.  源代码加密. `pyportable-installer` 内置了多种编译和混淆器供选择, 其中推荐使用 [pyportable-crypto](https://github.com/likianta/pyportable-crypto) 库或 [pyarmor](https://github.com/dashingsoft/pyarmor) 库对源代码进行混淆, 保障代码安全\n\n5.  开箱即用. `pyportable-installer` 打包后的目录结构非常清晰, 如下示例:\n\n    ![](.assets/20210914-105505.png)\n\n    ```\n    my_project\n    |= dist\n       |= hello_world_0.1.0  # 打包结果\n          |= build  # 一些构建信息\n             |- mainifest.json\n          |= src    # 您的源代码将被编译并放置在此目录下\n             |- ...\n          |= lib    # 一些自定义的第三方库会放在此目录下, 并在程序启动时加载\n             |= pyportable_runtime  # 用于解算加密后的源代码, 保障代码安全\n                |- ...\n          |= venv   # 自带的虚拟环境 (可选)\n             |- ...\n          |- README.html    # 自述文档\n          |- Hello.exe      # 双击即可启动!\n    ```\n\n6.  不破坏相对路径. 在打包后的 `src` 目录下, 所有文件夹仍然维持着原项目的目录结构. 程序在启动时会将工作目录切换到启动脚本所在的目录, 这意味着您在原项目中所使用的相对路径, 在打包后仍然保持一致\n\n7.  摆脱虚拟环境 (该特性仍处于实验阶段). 您可以在 配置文件/虚拟环境配置项 中启用 "[depsland][1]" 选项, 客户端在安装 depsland 软件后, 运行您发布的程序时会自动部署依赖, 这样真正做到打包体积控制, 加快分发并减轻用户下载负担\n\n8.  *无痛更新 (该特性将在后续版本提供). 双击软件目录下的 `checkup/update.pyd` 即可获取软件的最新版本*\n\n9.  *激活和授权 (该特性将在后续版本提供). 该特性由 pyarmor 提供, `pyportable-installer` 将其同样整合在 all-in-one 配置文件中*\n\n# 工作流程\n\n它的流程可以概括如下:\n\n1.  准备您要打包的项目\n\n2.  在项目的根目录下新建一个 all-in-one 配置文件: "pyproject.json"\n\n    1.  该文件名是可以任取的\n\n    2.  [这里](./pyportable_installer/template/pyproject.json) 有一个模板文件可供使用. 以及一个 [手册](./docs/pyproject-template-manual.zh.md) 供查阅每个选项的格式和作用\n\n3.  通过 `pyportable-installer` 处理此配置文件, 完成打包:\n\n    ```py\n    from pyportable_installer import full_build\n    full_build(\'./pyproject.json\')\n    ```\n\n`pyportable-installer` 会为您的项目生成:\n\n> 注: 这里演示的是启用 `pyportable-crypto` 选项加密的效果.\n\n1.  加密后的源代码文件\n\n    1.  加密后的文件后缀仍然是 \'.py\'\n\n    2.  加密后的文件由 `~/lib/pyportable_runtime` 包在运行时解码\n\n    3.  使用文本编辑器打开加密文件, 其密文如下所示:  \n\n        ![](.assets/20210914-112055.png)\n\n2.  一个 exe 格式的启动器\n\n3.  自定义的启动器图标 (注: 缺省图标为 python.ico)\n\n4.  一个干净的虚拟环境 (这是可选的)\n\n5.  整个打包后的结果会以文件夹的形式存在\n\n之后, 您可以将该文件夹制作为压缩文件, 并作为 "免安装版" 的软件发布.\n\n# 安装和使用\n\n通过 pip 安装:\n\n```\npip install pyportable-installer\n```\n\n> 注意事项:\n> \n> 1. pyportable-installer 最新版本为 4.3.0+ (截至 2021-12-30)\n> 2. 上代版本 (3.3.3 及以前) 已不推荐使用\n> 3. pyportable-installer 需要 Python 3.8 及以上的解释器运行\n\n下面以一个 "Hello World" 项目为例, 介绍具体的打包工作:\n\n假设 "Hello World" 的项目结构如下:\n\n```\nhello_world_project\n|= data\n   |- names.txt\n|= dist\n|= hello_world\n   |- main.py\n   |  # def say_hello(file):\n   |  #     with open(file, \'r\') as f:\n   |  #         for name in f:\n   |  #             print(f\'Hello {name}!\')\n   |  #\n   |  # if __name__ == \'__main__\':\n   |  #     say_hello(\'../data/names.txt\')\n|- pyproject.json\n|- README.md\n```\n\n在项目根目录下新建 \'pyproject.json\' ([这里](./pyportable_installer/template/pyproject.json) 有一个模板文件可供使用), 填写以下内容:\n\n> 注: \'◆◇◆\' 标记的是必填的项, 其他可使用缺省设置.\n\n```json5\n{\n    // 注意: 凡是配置项中涉及到路径的填写, 均使用相对于本文件的相对路径, 或者使\n    // 用绝对路径. pyportable-installer 会自动将它们转换到打包目录的内部结构中.\n\n    // ◆◇◆ 应用名称 ◆◇◆\n    "app_name": "Hello World",\n    // ◆◇◆ 应用版本 ◆◇◆\n    "app_version": "0.1.0",\n    "description": "",\n    "authors": [],\n    "build": {\n        // ◆◇◆ 项目目录的入口 ◆◇◆\n        "proj_dir": "./hello_world",\n        // ◆◇◆ 打包结果放在哪里 ◆◇◆\n        //     注意打包结果的父目录必须事先存在, 以及打包结果目录事先应不存在.\n        //     否则打包活动将中止.\n        "dist_dir": "./dist/{app_name_snake}_{app_version}",\n        // ◆◇◆ 启动器配置 ◆◇◆\n        //  1. 启动器配置是一个字典.\n        //  2. 字典的键是要生成的启动器. 键可以使用花括号语法指代一个名称, 例如 \n        //     "{app_name}" 指代 "Hello World", "{app_name_lower}" 指代 "hello \n        //     world" 等 (详见配置指南).\n        //  3. 字典的值是该启动器的配置.\n        "launchers": {\n            "{app_name}": {\n                "file": "./hello_world/main.py",\n                "icon": "",\n                "function": "main",\n                "args": ["../data/names.txt"],\n                "kwargs": {}\n            }\n        },\n        "readme": "",\n        // ◆◇◆ 要加入到打包的附件 ◆◇◆\n        "attachments": {\n            "./data/names.txt": "assets"\n        },\n        "attachments_exclusions": [],\n        "attachments_exist_scheme": "override",\n        "module_paths": [],\n        "module_paths_scheme": "translate",\n        // ◆◇◆ 目标应用的 python 解释器版本 ◆◇◆\n        "python_version": "3.8",\n        "venv": {\n            "enable_venv": true,\n            "mode": "source_venv",\n            "options": {\n                "depsland": {\n                    "venv_name": "{app_name_lower}_venv",\n                    "venv_id": "",\n                    "requirements": [],\n                    "offline": false,\n                    "local": ""\n                },\n                "source_venv": {\n                    "path": "",\n                    "copy_venv": true\n                },\n                "pip": {\n                    "requirements": [],\n                    "pypi_url": "https://pypi.python.org/simple/",\n                    "offline": false,\n                    "local": ""\n                },\n                "embed_python": {\n                    "path": ""\n                }\n            }\n        },\n        "compiler": {\n            "name": "pyportable_crypto",\n            "options": {\n                "cythonize": {\n                    "c_compiler": "msvc",\n                    "python_path": "auto_detect"\n                },\n                "pyarmor": {\n                    "license": "trial",\n                    "obfuscate_level": 0\n                },\n                "pyc": {\n                    "optimize_level": 0\n                },\n                "pyportable_crypto": {\n                    "key": "{random}"\n                },\n                "zipapp": {\n                    "password": ""\n                }\n            }\n        },\n        "experimental_features": {\n            "add_pywin32_support": false,\n            "platform": "system_default"\n        },\n        "enable_console": true\n    },\n    "note": "",\n    "pyportable_installer_version": "4.3.0"\n}\n```\n\n*注: 更多用法请参考 [Pyproject Template Manual](./docs/pyproject-template-manual.zh.md).*\n\n运行以下代码即可生成打包结果:\n\n```py\nfrom pyportable_installer import full_build\nfull_build(\'pyproject.json\')\n\n# 当增量更新时, 运行以下:\n# from pyportable_installer import min_build\n# min_build(\'pyproject.json\')\n\n# 如不需要加密源代码, 运行以下 (仅用于调试!):\n# from pyportable_installer import debug_build\n# debug_build(\'pyproject.json\')\n```\n\n生成的安装包位于 `hello_world/dist/hello_world_0.1.0`:\n\n```\nhello_world\n|= dist\n   |= hello_world_0.1.0\n      |= build\n         |- manifest.json   # 1. 应用构建信息 (数据已自动去敏, 可随项目一起发布)\n      |= src\n         |= .pylauncher_conf\n            |- __main__.pkl     # 5. pylauncher 会从这里读取启动配置信息\n         |= data\n            |- names.txt\n         |= hello_world\n            |- main.py          # 2. 这是加密后的脚本, 与源文件同名\n         |- pylauncher.py       # 4. 启动器 (exe) 会调用这个文件\n      |= lib\n         |= pyportable_runtime\n            |- __init__.py\n            |- inject.pyd\n      |- Hello World.exe    # 3. 双击启动!\n|- ...\n```\n\n# 注意事项\n\n1.  要生成的打包目录事先应不存在, 否则 pyportable-installer 会中止打包\n2.  如果您启用了虚拟环境选项, 则安装路径不能包含中文, 否则会导致启动失败 (该问题可能与 Embedded Python 解释器有关)\n3.  `pyportable-installer` 需要 Python 3.8 及以上的解释器运行. 打包的目标 Python 版本可调 (目前对 Python 2.7 和 Python 3.7 以下的版本未做充分测试)\n\n# FAQ\n\n## 程序在启动后会出现一个黑色的控制台, 该怎么隐藏?\n\n在配置文件中将 "enable_console" 选项关闭:\n\n```json\n{\n    "...": "...",\n    "build": {\n        "...": "...",\n        "enable_console": false\n    }\n}\n```\n\n## 运行报错: 没有找到 tkinter 库\n\n这是因为您发布的项目中的虚拟环境使用的是嵌入式 Python, 而嵌入式 Python 并没有自带 tkinter 库.\n\n~~解决方法 1 (适用于 v4.0.0b3+): 在配置文件中的实验性功能中启用 "add_tkinter_support":~~\n\n```json5\n{\n    // ...\n    "build": {\n        // ...\n        "experimental_features": {\n            "add_tkinter_support": {\n                "enable": true,\n                "system_python_path": "_auto_detect"\n            }\n        }\n    }\n}\n```\n\n解决方法 2: 请参考此文: [如何将 Tk 套件加入到嵌入式 Python 中](./docs/add-tkinter-to-embed-python.md).\n\n## pywin32, win32, win32clipboard 等相关问题\n\n解决方法 1 (适用于 v4.0.0b4+): 在配置文件中的实验性功能中启用 "add_pywin32_support".\n\n解决方法 2: 请参考此文: [Pywin32 库相关问题产生原因及解决方法](./docs/pywin32-problems-and-solutions.zh.md).\n\n## 如何添加图标文件\n\n图标仅支持 ico 格式. 如果您手上只有 png, jpg 等格式的图片文件, 可通过在线网站转换:\n\n- https://www.aconvert.com/cn/image/\n\n或者通过本项目自带的脚本:\n\n*(注意: 需要事先安装 pillow 库.)*\n\n```python\nfrom pyportable_installer.bat_2_exe import png_2_ico\n\n# a) 传入文件名和目标位置\npng_2_ico.main(\'some_icon.png\', \'launcher.ico\')\n\n# b) 根据提示传入一个文件名, 会在同一目录下生成 .ico 后缀的同名文件\npng_2_ico.dialog()\n```\n\n## 如何通过配置文件实现: 打包路径用英文, 启动器名字用中文?\n\n更改配置文件如下所示:\n\n方案 1:\n\n```json5\n{\n    "app_name": "你好世界",  // 定义应用名为中文. 将生成 "你好世界.exe"\n    "build": {\n        "dist_dir": "dist/hello_world_{app_version}",  // 这里用英文路径\n        // ...\n    },\n    // ...\n}\n```\n\n方案 2 (推荐):\n\n```json5\n{\n    "app_name": "Hello World",  // 仍保持原来的名字\n    "build": {\n        "launchers": {\n            // 单独修改启动器的名字\n            "你好世界": {...}\n        },\n        // ...\n    },\n    // ...\n}\n```\n\n## pyportable-installer 加密的安全吗?\n\npyportable-installer 以插件的方式集成了多个编译器支持. 您可以在 `配置文件:compiler:name` 中选择合适的加密方案.\n\n完整的支持列表和各编译器差异可在手册 (*TODO*) 中查询. 开发者也可以定义自己的编译器来保障安全 (需要继承 `pyportable_installer.compilers.base_compiler.BaseCompiler`).\n\n这里简要介绍下推荐的编译器:\n\n**[pyportable-crypto](https://github.com/likianta/pyportable-crypto)**\n\npyportable-installer 的姊妹项目之一, 提供开源免费的加密措施.\n\npyportable-crypto 使用随机步骤生成器产生一个密钥机, 再将密钥机通过 cython 编译为 pyd 文件 (inject.pyd). 由于密钥机内置了密钥 (非固定码文保存), 在发布后附带此密钥机可以解算经同一密钥加密的 py 文件.\n\n值得注意的是 pyportable-crypto 的开发仍处于非常早期的阶段, 任何问题和反馈都欢迎提交到 issues. 在正式版推出之前, 建议使用下面要介绍的 pyarmor 库.\n\n**[pyarmor](https://github.com/dashingsoft/pyarmor)**\n\n阅读此文了解: [PyArmor 的安全性](https://pyarmor.readthedocs.io/zh/latest/security.html).\n\n注意: 请勿使用 pyarmor 试用版加密您的产品! 任何人都可以通过公开的 license 获取到源代码内容. 如需在正式产品中使用 pyarmor, 请 [购买 pyarmor 使用许可](https://pyarmor.readthedocs.io/zh/latest/license.html).\n\n## 虚拟环境各个配置方案的区别, 我该如何选择?\n\n*TODO*\n\n**[depsland][2]**\n\n*TODO*\n\n[1]: https://github.com/likianta/depsland/releases\n[2]: https://github.com/likianta/depsland\n',
    'author': 'Likianta',
    'author_email': 'likianta@foxmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
